@@ -27,16 +27,12 @@
 
 #include <algorithm>
 
+using namespace Qt::Literals::StringLiterals;
+
 class LocalFileListingTests: public QObject, public DatabaseTestData
 {
     Q_OBJECT
 
-public:
-
-    LocalFileListingTests(QObject *parent = nullptr) : QObject(parent)
-    {
-    }
-
 private Q_SLOTS:
 
     void initTestCase()
@@ -147,10 +143,28 @@ private Q_SLOTS:
         auto firstNewTracks = firstNewTracksSignal.at(0).value<DataTypes::ListTrackDataType>();
         const auto &secondNewTracksSignal = tracksListSpy.at(1);
         auto secondNewTracks = secondNewTracksSignal.at(0).value<DataTypes::ListTrackDataType>();
-        auto newCovers = secondNewTracksSignal.at(1).value<QHash<QString, QUrl>>();
 
         QCOMPARE(firstNewTracks.count() + secondNewTracks.count(), 5);
-        QCOMPARE(newCovers.count(), 5);
+
+        tracksListSpy.clear();
+        removedTracksListSpy.clear();
+
+        myListing.resetAndRefreshContent();
+
+        QCOMPARE(tracksListSpy.count(), 1);
+        QCOMPARE(removedTracksListSpy.count(), 0);
+
+        auto newTracks = tracksListSpy.at(0).at(0).value<DataTypes::ListTrackDataType>();
+
+        QCOMPARE(newTracks.count(), 5);
+
+        tracksListSpy.clear();
+        removedTracksListSpy.clear();
+
+        myListing.refreshContent();
+
+        QVERIFY(tracksListSpy.isEmpty());
+        QVERIFY(removedTracksListSpy.isEmpty());
     }
 
     void addAndRemoveTracks()
@@ -204,10 +218,8 @@ private Q_SLOTS:
 
         auto newTracksSignal = tracksListSpy.at(0);
         auto newTracks = newTracksSignal.at(0).value<DataTypes::ListTrackDataType>();
-        auto newCovers = newTracksSignal.at(1).value<QHash<QString, QUrl>>();
 
         QCOMPARE(newTracks.count(), 1);
-        QCOMPARE(newCovers.count(), 1);
 
         QDir musicDirectory(musicPath);
         QCOMPARE(musicDirectory.removeRecursively(), true);
@@ -239,10 +251,8 @@ private Q_SLOTS:
 
         auto newTracksSignalLast = tracksListSpy.at(1);
         auto newTracksLast = newTracksSignalLast.at(0).value<DataTypes::ListTrackDataType>();
-        auto newCoversLast = newTracksSignalLast.at(1).value<QHash<QString, QUrl>>();
 
         QCOMPARE(newTracksLast.count(), 1);
-        QCOMPARE(newCoversLast.count(), 1);
     }
 
     void addTracksAndRemoveDirectory()
@@ -298,10 +308,8 @@ private Q_SLOTS:
 
         auto newTracksSignal = tracksListSpy.at(0);
         auto newTracks = newTracksSignal.at(0).value<DataTypes::ListTrackDataType>();
-        auto newCovers = newTracksSignal.at(1).value<QHash<QString, QUrl>>();
 
         QCOMPARE(newTracks.count(), 1);
-        QCOMPARE(newCovers.count(), 1);
 
         QDir musicDirectory(innerMusicPath);
         musicDirectory.removeRecursively();
@@ -333,10 +341,8 @@ private Q_SLOTS:
 
         auto newTracksSignalLast = tracksListSpy.at(1);
         auto newTracksLast = newTracksSignalLast.at(0).value<DataTypes::ListTrackDataType>();
-        auto newCoversLast = newTracksSignalLast.at(1).value<QHash<QString, QUrl>>();
 
         QCOMPARE(newTracksLast.count(), 1);
-        QCOMPARE(newCoversLast.count(), 1);
     }
 
     void addAndMoveTracks()
@@ -402,10 +408,8 @@ private Q_SLOTS:
 
         auto newTracksSignal = tracksListSpy.at(0);
         auto newTracks = newTracksSignal.at(0).value<DataTypes::ListTrackDataType>();
-        auto newCovers = newTracksSignal.at(1).value<QHash<QString, QUrl>>();
 
         QCOMPARE(newTracks.count(), 1);
-        QCOMPARE(newCovers.count(), 1);
 
         QDir().rename(musicPath, musicFriendPath);
 
@@ -439,10 +443,8 @@ private Q_SLOTS:
 
         auto newTracksSignalLast = tracksListSpy.at(1);
         auto newTracksLast = newTracksSignalLast.at(0).value<DataTypes::ListTrackDataType>();
-        auto newCoversLast = newTracksSignalLast.at(1).value<QHash<QString, QUrl>>();
 
         QCOMPARE(newTracksLast.count(), 1);
-        QCOMPARE(newCoversLast.count(), 1);
     }
 
     void restoreRemovedTracks()
@@ -483,14 +485,16 @@ private Q_SLOTS:
         QCOMPARE(indexingFinishedSpy.count(), 0);
         QCOMPARE(askRestoredTracksSpy.count(), 1);
 
-        myListing.restoredTracks({{QUrl::fromLocalFile(QStringLiteral("/removed/files1")), QDateTime::fromMSecsSinceEpoch(1)},
-                                  {QUrl::fromLocalFile(QStringLiteral("/removed/files2")), QDateTime::fromMSecsSinceEpoch(2)}});
+        myListing.setIndexedTracks({
+            {QUrl::fromLocalFile(QStringLiteral("/removed/files1")), QDateTime::fromMSecsSinceEpoch(1)},
+            {QUrl::fromLocalFile(QStringLiteral("/removed/files2")), QDateTime::fromMSecsSinceEpoch(2)},
+        });
 
         QCOMPARE(tracksListSpy.count(), 0);
         QCOMPARE(removedTracksListSpy.count(), 1);
         QCOMPARE(modifiedTracksListSpy.count(), 0);
         QCOMPARE(indexingStartedSpy.count(), 1);
-        QCOMPARE(indexingFinishedSpy.count(), 0);
+        QCOMPARE(indexingFinishedSpy.count(), 1);
         QCOMPARE(askRestoredTracksSpy.count(), 1);
 
         auto removedTracksSignal = removedTracksListSpy.at(0);
@@ -506,6 +510,148 @@ private Q_SLOTS:
         QCOMPARE(removedTracks[0], QUrl::fromLocalFile(QStringLiteral("/removed/files1")));
         QCOMPARE(removedTracks[1], QUrl::fromLocalFile(QStringLiteral("/removed/files2")));
     }
+
+    void refreshIndex()
+    {
+        LocalFileListing myListing;
+
+        const QString musicOriginPath = QStringLiteral(LOCAL_FILE_TESTS_SAMPLE_FILES_PATH) + u"/music"_s;
+
+        const QString musicParentPath = QStringLiteral(LOCAL_FILE_TESTS_WORKING_PATH) + u"/music2"_s;
+
+        const QString musicPath = musicParentPath + u"/data/innerData"_s;
+
+        const QString trackOggPath = musicPath + u"/test.ogg"_s;
+
+        const QString trackM4aPath = musicPath + u"/test.m4a"_s;
+
+        const QString trackMp3Path = musicPath + u"/test.mp3"_s;
+
+        QDir musicParentDirectory(musicParentPath);
+        QDir musicDirectory(musicPath);
+        QFile trackOgg(musicOriginPath + u"/test.ogg"_s);
+        QFile trackMp3(musicOriginPath + u"/test.mp3"_s);
+        QFile trackM4a(musicOriginPath + u"/test.m4a"_s);
+
+        QVERIFY(musicParentDirectory.removeRecursively());
+
+        QVERIFY(!musicDirectory.exists());
+        QVERIFY(musicDirectory.mkpath(musicPath));
+        QVERIFY(musicDirectory.exists());
+        QVERIFY(musicDirectory.isEmpty());
+        QVERIFY(trackOgg.copy(trackOggPath));
+        QVERIFY(trackMp3.copy(trackMp3Path));
+        QVERIFY(trackM4a.copy(trackM4aPath));
+
+        QSignalSpy tracksListSpy(&myListing, &LocalFileListing::tracksList);
+        QSignalSpy removedTracksListSpy(&myListing, &LocalFileListing::removedTracksList);
+        QSignalSpy modifiedTracksListSpy(&myListing, &LocalFileListing::modifyTracksList);
+        QSignalSpy errorWatchingFileSystemChangesSpy(&myListing, &LocalFileListing::errorWatchingFileSystemChanges);
+
+        QCOMPARE(tracksListSpy.count(), 0);
+        QCOMPARE(removedTracksListSpy.count(), 0);
+        QCOMPARE(modifiedTracksListSpy.count(), 0);
+
+        myListing.init();
+
+        QCOMPARE(tracksListSpy.count(), 0);
+        QCOMPARE(removedTracksListSpy.count(), 0);
+        QCOMPARE(modifiedTracksListSpy.count(), 0);
+
+        myListing.setAllRootPaths({musicParentPath});
+
+        QCOMPARE(tracksListSpy.count(), 0);
+        QCOMPARE(removedTracksListSpy.count(), 0);
+        QCOMPARE(modifiedTracksListSpy.count(), 0);
+
+        QCOMPARE(myListing.allRootPaths(), QStringList{musicParentPath});
+
+        myListing.refreshContent();
+
+        QCOMPARE(tracksListSpy.count(), 2);
+        QCOMPARE(removedTracksListSpy.count(), 0);
+        QCOMPARE(modifiedTracksListSpy.count(), 0);
+
+        auto newTracks = tracksListSpy.at(0).at(0).value<DataTypes::ListTrackDataType>();
+
+        QCOMPARE(newTracks.count(), 2);
+
+        newTracks = tracksListSpy.at(1).at(0).value<DataTypes::ListTrackDataType>();
+
+        QCOMPARE(newTracks.count(), 1);
+
+        tracksListSpy.clear();
+
+        myListing.refreshContent();
+
+        QCOMPARE(tracksListSpy.count(), 0);
+        QCOMPARE(removedTracksListSpy.count(), 0);
+        QCOMPARE(modifiedTracksListSpy.count(), 0);
+
+        myListing.resetAndRefreshContent();
+
+        QCOMPARE(tracksListSpy.count(), 1);
+        QCOMPARE(removedTracksListSpy.count(), 0);
+        QCOMPARE(modifiedTracksListSpy.count(), 0);
+
+        newTracks = tracksListSpy.at(0).at(0).value<DataTypes::ListTrackDataType>();
+
+        QCOMPARE(newTracks.count(), 3);
+
+        tracksListSpy.clear();
+
+        // Set the file modified time to some point in the future to ensure
+        // the file does not get re-indexed
+        myListing.setIndexedTracks({
+            {QUrl::fromLocalFile(trackOggPath), QDateTime::currentDateTime().addYears(1)}
+        });
+
+        QCOMPARE(tracksListSpy.count(), 1);
+        QCOMPARE(removedTracksListSpy.count(), 0);
+        QCOMPARE(modifiedTracksListSpy.count(), 0);
+
+        newTracks = tracksListSpy.at(0).at(0).value<DataTypes::ListTrackDataType>();
+
+        QCOMPARE(newTracks.count(), 2);
+
+        tracksListSpy.clear();
+
+        myListing.setAllRootPaths({musicParentPath + u"/"_s});
+
+        myListing.refreshContent();
+
+        QCOMPARE(tracksListSpy.count(), 0);
+        QCOMPARE(removedTracksListSpy.count(), 0);
+        QCOMPARE(modifiedTracksListSpy.count(), 0);
+
+        myListing.resetAndRefreshContent();
+
+        QCOMPARE(tracksListSpy.count(), 1);
+        QCOMPARE(removedTracksListSpy.count(), 0);
+        QCOMPARE(modifiedTracksListSpy.count(), 0);
+
+        newTracks = tracksListSpy.at(0).at(0).value<DataTypes::ListTrackDataType>();
+
+        QCOMPARE(newTracks.count(), 3);
+
+        tracksListSpy.clear();
+
+        myListing.setIndexedTracks({
+            {QUrl::fromLocalFile(trackM4aPath), QDateTime::currentDateTime().addYears(1)},
+            {QUrl::fromLocalFile(trackMp3Path), QDateTime::fromMSecsSinceEpoch(1)},
+            {QUrl::fromLocalFile(u"/does/not/exist.mp3"_s), QDateTime::currentDateTime()},
+        });
+
+        QCOMPARE(tracksListSpy.count(), 1);
+        QCOMPARE(removedTracksListSpy.count(), 1);
+        QCOMPARE(modifiedTracksListSpy.count(), 0);
+
+        newTracks = tracksListSpy.at(0).at(0).value<DataTypes::ListTrackDataType>();
+        auto removedTracks = removedTracksListSpy.at(0).at(0).value<QList<QUrl>>();
+
+        QCOMPARE(newTracks.count(), 2);
+        QCOMPARE(removedTracks.count(), 1);
+    }
 };
 
 QTEST_GUILESS_MAIN(LocalFileListingTests)
